#!/usr/bin/env python
# encoding: utf-8
"""
Author: Nick Bearson, nickb@ssec.wisc.edu
Copyright (c) 2012 University of Wisconsin SSEC. All rights reserved.
"""


def get_iff_attrs(instrument):
  IFF_ATTRS = {
    'platform'            : instrument.PLATFORM,
    'sensor'              : instrument.SENSOR,
    'software_version'    : instrument.SOFTWARE_VERSION,
    'original_file_names' : instrument.FILENAMES,
    'creation_date'       : instrument.CREATION_DATE,
    'granule_start_time'  : instrument.START_TIME,
    'granule_end_time'    : instrument.END_TIME,
    'creation_host'       : instrument.CREATION_HOST,
  }
  return IFF_ATTRS


def get_cm_struct(instrument):
  CM_STRUCT = {
    'Latitude' : {
      'source_function' : instrument.Latitude,
      'attrs' : {
        'long_name' : 'Geodetic latitude for the center of the pixel',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  
    'Longitude' : {
      'source_function' : instrument.Longitude,
      'attrs' : {
        'long_name' : 'Geodetic longitude for the center of the pixel',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  }

  for i in range(1,7):
    dsname = 'Cloudmask Byte %d' % (i)
    CM_STRUCT[dsname] = {}
    CM_STRUCT[dsname]['source_function'] = instrument.Cloudmask(i)
    CM_STRUCT[dsname]['attrs'] = {}
    CM_STRUCT[dsname]['dims'] = ['line', 'element']

  CM_STRUCT['Quality Assurance'] = {
      'source_function' : instrument.CMQA,
      'attrs' : {},
      'dims' : ['line', 'element', 'byte'],    
  }

  return CM_STRUCT


def get_iff_struct(instrument):
  IFF_STRUCT = {
    'ReflectiveSolarBands'   : {
      'source_function' : instrument.ReflectiveSolarBands,
      'attrs' : {
        'long_name' : 'Earth view reflective solar bands, not corrected for solar zenith angle',
        'units' : 'None',
        'band_names' : instrument.ReflectiveBandNums,
      },
      'dims' : ['reflective_band', 'line', 'element'],
    },
  
    'ReflectiveBandCenters'  : {
      'source_function' : instrument.ReflectiveBandCenters,
      'attrs' : {
        'long_name' : 'Center wavelength of the reflective solar bands',
        'units' : 'microns',
      },
      'dims' : ['reflective_band'],
    },
  
    'EmissiveBands'   : {
      'source_function' : instrument.EmissiveBands,
      'attrs' : {
        'long_name' : 'Earth view thermal emissive bands',
        'units' : 'W m-2 um-1 sr-1',
        'band_names' : instrument.EmissiveBandNums,
        'missing_value' : -999.0,
        '_FillValue' : -999.0,
        },
      'dims' : ['emissive_band', 'line', 'element'],
    },
  
    'EmissiveBandCenters' : {
      'source_function' : instrument.EmissiveBandCenters,
      'attrs' : {
        'long_name' : 'Center wavelength of the thermal emissive bands',
        'units' : 'microns',
      },
      'dims' : ['emissive_band'],
    },

    'BrightnessTemperatureBands'   : {
      'source_function' : instrument.BrightnessTemperatureBands,
      'attrs' : {
        'long_name' : 'Earth view thermal emissive bands converted into brightness temperatures',
        'units' : 'K',
        'band_names' : instrument.BrightnessTemperatureBandNums,
        'missing_value' : -999.0,
        '_FillValue' : -999.0,
        },
      'dims' : ['bt_band', 'line', 'element'],
    },
  
    'BrightnessTemperatureBandCenters' : {
      'source_function' : instrument.BrightnessTemperatureBandCenters,
      'attrs' : {
        'long_name' : 'Center wavelength of the thermal emissive bands',
        'units' : 'microns',
      },
      'dims' : ['bt_band'],
    },

    'Height' : {
      'source_function' : instrument.Height,
      'attrs' : {
        'long_name' : 'Geodetic heights above geoid',
        'units' : 'meters',
      },
      'dims' : ['line', 'element'],
    },
  
    'Range' : {
      'source_function' : instrument.Range,
      'attrs' : {
        'long_name' : 'Slant range',
        'units' : 'meters',
      },
      'dims' : ['line', 'element'],
    },
  
    'SolarZenith' : {
      'source_function' : instrument.SolarZenith,
      'attrs' : {
        'long_name' : 'Solar zenith angles for the corresponding EV frames',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  
    'SolarAzimuth' : {
      'source_function' : instrument.SolarAzimuth,
      'attrs' : {
        'long_name' : 'Solar azimuth angles for the corresponding EV frames',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  
    'SensorZenith' : {
      'source_function' : instrument.SensorZenith,
      'attrs' : {
        'long_name' : 'Instrument zenith angles for the corresponding EV frames',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  
    'SensorAzimuth' : {
      'source_function' : instrument.SensorAzimuth,
      'attrs' : {
        'long_name' : 'Instrument azimuth angles for the corresponding EV frames',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  
    'Latitude' : {
      'source_function' : instrument.Latitude,
      'attrs' : {
        'long_name' : 'Geodetic latitude for the center of the pixel',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  
    'Longitude' : {
      'source_function' : instrument.Longitude,
      'attrs' : {
        'long_name' : 'Geodetic longitude for the center of the pixel',
        'units' : 'degrees',
      },
      'dims' : ['line', 'element'],
    },
  
    'ScanStartTime' : {
      'source_function' : instrument.ScanStartTime,
      'attrs' : {
        'long_name' : 'Earth view scan start time',
        'units' : 'seconds',
      },
      'dims' : ['line'],
    },

    'SounderLineIndex' : {
      'source_function' : instrument.SounderLineIndex,
      'attrs' : {
        'long_name' : 'Index of sounder line used when collocating',
        'units' : 'None',
      },
      'dims' : ['line', 'element'],
    },

    'SounderElementIndex' : {
      'source_function' : instrument.SounderElementIndex,
      'attrs' : {
        'long_name' : 'Index of sounder element used when collocating',
        'units' : 'None',
      },
      'dims' : ['line', 'element'],
    },

    'SounderCloudHeights' : {
      'source_function' : instrument.SounderCloudHeights,
      'attrs' : {
        'long_name' : 'cloud_top_height field from HIRS CTP CDR files',
        'units' : 'm',
      },
      'dims' : ['line', 'element'],
    },
    'SounderCloudPressure' : {
      'source_function' : instrument.SounderCloudPressure,
      'attrs' : {
        'long_name' : 'cloud_top_pressure field from HIRS CTP CDR files',
        'units' : 'hPa',
      },
      'dims' : ['line', 'element'],
    },
    'SounderCloudTemperature' : {
      'source_function' : instrument.SounderCloudTemperature,
      'attrs' : {
        'long_name' : 'cloud_top_temperature field from HIRS CTP CDR files',
        'units' : 'K',
      },
      'dims' : ['line', 'element'],
    },
  
    'HirsMask' : {
      'source_function' : instrument.HirsMask,
      'attrs' : {
        'long_name' : 'Flag indicating whether this pixel is overlapped by an actual (i.e. not interpolated etc.) HIRS observation',
        'units' : 'None',
      },
      'dims' : ['line', 'element'],
    },

#    'LandSeaMask' : {
#      'source_function' : instrument.LandSeaMask,
#      'attrs' : {
#        'long_name' : 'Land Sea Mask',
#        'units' : 'None',
#      },
#      'dims' : ['line', 'element'],
#    },

#    'LandSeaMask_Compare' : {
#      'source_function' : instrument.LandSeaMask_compare,
#      'attrs' : {
#        'long_name' : 'Land Sea Mask MYD03',
#        'units' : 'None',
#      },
#      'dims' : ['line', 'element'],
#    },

  }

  return IFF_STRUCT

